@@ -1,31 +1,32 @@
 from execelasticsearch.connection import Clients
+from execelasticsearch.mapping import MappingData
 from elasticsearch import Elasticsearch, helpers, exceptions
 from json import loads, dumps
 
 
 def _pack(body):
-    body = dumps(body).replace('{', '{{').replace('}', '}}').replace('{{}}', '{}')
-    return lambda *args: loads(body.format(*args))
+    body = dumps(body).replace('{', '{{').replace('}', '}}').replace('<<', '{').replace('>>', '}')
+    return lambda **kwargs: loads(body.format(**kwargs))
 
 
 class SearchBody:
-    __body_list = dict(
-        exists=_pack({"query": {"bool": {"must": {"exists": {"field": "{}"}}}}}),
-        not_exists=_pack({"query": {"bool": {"must_not": {"exists": {"field": "{}"}}}}}),
-    )
+    __search_body = MappingData.SearchBody.Template
+    __body_list = {k: _pack(v) for k, v in __search_body.items()}
 
     def __setitem__(self, body_type: str, body_args: str):
+        self.__search_body.update({body_type.lower(): body_args})
         self.__body_list.update({body_type.lower(): _pack(body_args)})
 
     def __getitem__(self, body_type):
         return self.__body_list[body_type]
 
     def update(self, **kwargs):
+        self.__search_body.update({body_type.lower(): body_args for body_type, body_args in kwargs.items()})
         self.__body_list.update({body_type.lower(): _pack(body_args) for body_type, body_args in kwargs.items()})
 
     @property
     def body_type_list(self):
-        return list(self.__body_list.keys())
+        return "\n"+"\n".join([dumps({k: v}) for k, v in self.__search_body.items()])+"\n"
 
 
 class ExecES:
@@ -69,20 +70,21 @@ class ExecES:
             self.__clients[host], actions=[{'_op_type': 'delete', '_id': i} for i in ids],
             index=index, chunk_size=chunk_size, ignore_status=ignore_status, **kwargs)]) for host in set(hosts)}
 
-    def mget(self, index: str, ids: list, hosts: list, _source_includes=None, **kwargs):
+    def mget(self, index: str, ids: list, hosts: list, _source_includes: list = None, **kwargs):
         return {host: self.__mget(index, ids, host, _source_includes, **kwargs) for host in set(hosts)}
 
     def exists_ids(self, index: str, ids: list, hosts: list, **kwargs):
         return {host: list(self.__mget(index, ids, host, **kwargs).keys()) for host in set(hosts)}
 
-    def search(self, index: str, body_type: str, body_args: tuple, host: str, doc_type=None,
-               scroll='2m', size=1000, _source_includes=None, **kwargs) -> iter:
+    def search(self, index: str, body_kwargs: dict, host: str, doc_type=None,
+               scroll='2m', size=1000, _source_includes: list = None, **kwargs) -> iter:
         """
         body_type: execelasticsearch.handler.SearchBody().body_type_list will show default body_type.
         """
         source = True if _source_includes or kwargs.get('_source_excludes') else False
-        body = self.search_body[body_type](*body_args)
-        res = self.__clients[host].search(index=index, body=body, doc_type=doc_type or self.__doc_type(host),
+        body_type, = body_kwargs
+        body = self.search_body[body_type](**body_kwargs[body_type])
+        res = self.__clients[host].search(index=index, body=body, doc_type=doc_type,
                                           scroll=scroll, size=size, _source=source,
                                           _source_includes=_source_includes, **kwargs)
         for data in self.__scroll_data(res, host):
@@ -90,12 +92,12 @@ class ExecES:
 
     def __scroll_data(self, search_response, host):
         sid = search_response['_scroll_id']
-        scroll_size = search_response['hits']['total']['value']
-        while scroll_size > 0:
-            yield search_response['hits']['hits'] if search_response else []
+        scroll_data = search_response['hits']['hits']
+        while len(scroll_data) > 0:
+            yield scroll_data
             search_response = self.__clients[host].scroll(scroll_id=sid, scroll='2m')
             sid = search_response['_scroll_id']
-            scroll_size = len(search_response['hits']['hits'])
+            scroll_data = search_response['hits']['hits']
 
     def __update_or_ignore(self, index: str, id_, data: dict, host, doc_type=None, **kwargs):
         try:
